/***************************************************************************//**
 *   @file   Main.c
 *   @brief  Implementation of the program's main function.
 *   @author Dan Nechita
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 567
*******************************************************************************/

/******************************************************************************/
/* Include Files                                                              */
/******************************************************************************/
#include "YRDKRL78G13.h"        // YRDKRL78G13 definitions.
#include "ST7579.h"				// ST7579 definitions.
#include "AD5790.h"             // AD5790 definitions.

/******************************************************************************/
/* Option Bytes                                                               */
/******************************************************************************/
#pragma location = "OPTBYTE"
__root const uint8_t opbyte0 = 0x00U;
#pragma location = "OPTBYTE"
__root const uint8_t opbyte1 = 0xFFU;
#pragma location = "OPTBYTE"
__root const uint8_t opbyte2 = 0xE8U;
#pragma location = "OPTBYTE"
__root const uint8_t opbyte3 = 0x04U;

/******************************************************************************/
/* Security ID                                                                */
/******************************************************************************/
#pragma location = "SECUID"
__root const uint8_t secuid[10] = 
    {0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U};

/******************************************************************************/
/* Variables Declarations                                                     */
/******************************************************************************/
const unsigned char adiLogo [2 * 19] =
	{ 0xFF, 0xFF, 0xFF, 0x03, 0x03, 0x03, 0x07, 0x07,
	  0x0F, 0x0F, 0x1F, 0x1F, 0x3F, 0x3F, 0x7F, 0x7F,
	  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xC0, 0xC0,
	  0xC0, 0xE0, 0xE0, 0xF0, 0xF0, 0xF8, 0xF8, 0xFC,
	  0xFC, 0xFE, 0xFE, 0xFF, 0xFF, 0xFF };	// ADI Logo.
signed long dacVal   = 0;
signed short step    = 0;
unsigned long result = 0;

/***************************************************************************//**
 * @brief Creates a delay of miliseconds.
 *
 * @param mSeconds - time in miliseconds.
 *
 * @return None.
*******************************************************************************/
void DelayMs(unsigned long mSeconds)
{
    unsigned long time = 0;
    
    R_TAU0_Channel0_Start();
    while(time < mSeconds)
    {
        if(TMIF00 == 1)
        {
            time++;
            TMIF00 = 0;
        }
    }
    R_TAU0_Channel0_Stop();
}

/***************************************************************************//**
 * @brief Main function.
 *
 * @param None.
 *
 * @return None.
*******************************************************************************/
void  main(void)
{
    YRDKRL78G13_Init();
    ST7579_Init();
    ST7579_Image(2, 0, adiLogo, 19, 16);
    ST7579_String(2, 21, "ANALOG");
    ST7579_String(3, 21, "DEVICES");
    ST7579_String(4, 0, "wiki.analog.com");
    DelayMs(2000);
    ST7579_Clear();
    ST7579_Image(0, 0,adiLogo, 19, 16);	
    ST7579_String(0, 22, "ST7579 OK");    
    // Initializes communication with the device.
    if(AD5790_Init())
    {
        ST7579_String(1, 22, "AD5790 OK");
    }
    else
    {
        ST7579_String(1, 22, "AD5790 Err");
    }
    // Resets the device to its power-on state.
    AD5790_SoftInstruction(AD5790_SOFT_CTRL_RESET);
    // Enables the DAC output.
    AD5790_EnableOutput(1);
    // The DAC register is set to use offset binary coding.
    AD5790_Setup(AD5790_CTRL_BIN2SC);
    // Sets the value to which the DAC output is set when CLEAR is enabled.
    AD5790_SetRegisterValue(AD5790_REG_CLR_CODE, 
                            0x80000, 
                            3);  
    // Performs a soft CLEAR operation.
     AD5790_SoftInstruction(AD5790_SOFT_CTRL_CLR);
    // Reads and displays the internal registers.
    // Read DAC
    result = AD5790_GetRegisterValue(AD5790_REG_DAC, 3);
    result = (result & 0xFFFFF);
    ST7579_String(3, 0, "DAC:");
    ST7579_HexNumber(3, 70, result, 3);
    // Read Control
    result = AD5790_GetRegisterValue(AD5790_REG_CTRL, 3);
    result = (result & 0x3E);
    ST7579_String(4, 0, "CTRL:");
    ST7579_HexNumber(4, 70, result, 3);
    // Read ClearCode
    result = AD5790_GetRegisterValue(AD5790_REG_CLR_CODE, 3);
    result = (result & 0xFFFFF);
    ST7579_String(5, 0, "CODE:");
    ST7579_HexNumber(5, 70, result, 3);
    // Generates a triangle signal.
    while(1)
    { 
       if(dacVal >= (0x000C0000 - step))
       {
           step = -256;
       }
       else if(dacVal <= (0x00000000 - step))
            {
                step = 256;
            }
       dacVal += step;       
       AD5790_SetDacValue(dacVal);
       DelayMs(1);
    }
}
